import router from './router'
import store from './store'
import { Message } from 'element-ui'
import NProgress from 'nprogress' // progress bar
import 'nprogress/nprogress.css'// progress bar style
import { getToken,setToken } from '@/utils/auth' // getToken from cookie

NProgress.configure({ showSpinner: false })// NProgress Configuration

// permissiom judge function
function hasPermission(roles, permissionRoles) {
  if (roles.indexOf('admin') >= 0) return true // admin permission passed directly
  if (!permissionRoles) return true
  return roles.some(role => permissionRoles.indexOf(role) >= 0)
}

const whiteList = ['/login', '/authredirect']// no redirect whitelist
//自定义路由
const myRoles = [
  'couponsManage', 
  'couponsList', 

  'answerGame', 
  'answerList', 
  
  'raffleManage', 
  'raffleList', 
  'raffleDataList', 
  'raffleLogsList',

  'lotteryManage', 
  'lotteryList', 
  'lotteryDataList', 
  'lotteryLogsList',

  'goodsManage', 
  'goodsExchangeRules', 
  'goodsList', 

  'giftManage', 
  'giftExchangeRules', 
  'giftList', 

  'welfareManage', 
  'welfareList', 

  'exchangeManage', 
  'exchangeList',

  'cancelledList',
  'approvalList',
  'pasList',
  'voidList',

  'pointManage', 
  'pointIndateList',
  'pointInstructions', 
  'pointRulesList', 
  'pointList', 
  'pointsDetailList',
  'pointsPlusOrMinus',

  'activityManage', 
  'activityList', 

  'noticeManage', 
  'noticeList', 

  'ranking',
  'rankingList',

  'medalManage', 
  'medalList', 
  'medalDataList',

  'commendManage', 
  'commendList',
  'commendDataList', 
  'citeList',
  'operateCiteList',
  'ceoCiteList',

  'trainManage', 
  'upLoadFileRules',
  'trainList',
  'operateTrainList',
  'ceoTrainList',

  'festivalManage', 
  'festivalList', 

  'certManage',
  'certSetList', 
  'certRules',
  'certList', 

  'dictManage', 
  'dictList', 
  'dictDataList', 

  'postManage',
  'postList',
  'postApprovalList',

  'rankingManage',
  'answerRanking',
  'gameRanking',

  'content',
  'yearLottoNotice',
  'bannerManage'
]

router.beforeEach((to, from, next) => {
  NProgress.start() // start progress bar
  // store.dispatch('SetToken', 'y8evar5b5yecmr6hjrhyokxw5tiqizw9');
  // setToken('y8evar5b5yecmr6hjrhyokxw5tiqizw9');
  const path = to.path;
  const token = to.query.xToken;
  if (path.indexOf('auth') != -1 && token) { 
    store.dispatch('SetToken', token);
    setToken(token);
  }
  if (getToken()) { // determine if there has token
    /* has token*/
    if (to.path === '/login') {
      next({ path: '/' })
      NProgress.done() // if current page is dashboard will not trigger	afterEach hook, so manually handle it
    } else {
      if (store.getters.roles.length === 0) { // 判断当前用户是否已拉取完user_info信息
        store.dispatch('GetUserInfo').then(res => {
          store.dispatch('GetUserMenus').then(res => { // 拉取user_info
            const roles = res.data.data // note: roles must be a array! such as: ['editor','develop']
            // const roles = myRoles;
            store.dispatch('GenerateRoutes', { roles }).then(() => { // 根据roles权限生成可访问的路由表
              router.addRoutes(store.getters.addRouters) // 动态添加可访问路由表
              next({ ...to, replace: true }) // hack方法 确保addRoutes已完成 ,set the replace: true so the navigation will not leave a history record
              next({ path: '/' });
            })
          })
        }).catch(() => {
          store.dispatch('FedLogOut').then(() => {
            Message.error('Verification failed, please login again')
            const prodHref = 'http://dgt.dgtis.com/oneportal/login';//正式地址

            // const prodHref = 'http://dgtcloud.dgtis.com/oneportal/login';//阿里云地址
            
            const devHref = 'http://192.168.100.87:8080/oneportal/login';//测试地址

            location.href = process.env.NODE_ENV === 'production' ? prodHref : devHref;
            // next({ path: '/login' })
          })
        })
      } else {
        // 没有动态改变权限的需求可直接next() 删除下方权限判断 ↓
        if (hasPermission(store.getters.roles, to.meta.roles)) {
          next()//
        } else {
          next({ path: '/401', replace: true, query: { noGoBack: true }})
        }
        // 可删 ↑
      }
    }
  } else {
    /* has no token*/
    if (whiteList.indexOf(to.path) !== -1) { // 在免登录白名单，直接进入
      next()
    } else {
      const prodHref = 'http://dgt.dgtis.com/oneportal/login';//正式地址

      // const prodHref = 'http://dgtcloud.dgtis.com/oneportal/login';//阿里云地址
      
      const devHref = 'http://192.168.100.87:8080/oneportal/login';//测试地址

      location.href = process.env.NODE_ENV === 'production' ? prodHref : devHref;
      // next('/login') // 否则全部重定向到登录页
      NProgress.done() // if current page is login will not trigger afterEach hook, so manually handle it
    }
  }
})

router.afterEach(() => {
  NProgress.done() // finish progress bar
})
